<?php

namespace Models;

require_once '../models/Model.php';
use Controller\Connection;
use PDO;

class User extends Model
{
    protected $name;
    protected $mail;
    protected $password;
    protected $sector;
    protected $token;
    protected $mail_verified;

    public function __construct($name, $mail, $sector, $password)
    {
        $this->name = $name;
        $this->mail = $mail;
        $this->password = password_hash($password, PASSWORD_DEFAULT);
        $this->sector = $sector;
        $this->token = self::generateRandomString(20);
        $this->mail_verified = false;
    }

    public function save(Connection $connection)
    {
        $con = $connection->get_connection();

        $stmt = $con->prepare("INSERT INTO users (name, mail, password, sector, token, mail_verified) 
            VALUES (:name, :mail, :password, :sector, :token, :mail_verified)");

        $stmt->bindParam(":name", $this->name);
        $stmt->bindParam(":mail", $this->mail);
        $stmt->bindParam(":password", $this->password);
        $stmt->bindParam(":sector", $this->sector);
        $stmt->bindParam(":token", $this->token);
        $stmt->bindParam(":mail_verified", $this->mail_verified);

        return $stmt->execute();
    }

    public static function usersDetail(Connection $connection, string $email)
    {
        $con = $connection->get_connection();
        $stmt = $con->prepare("SELECT id, name, mail, sector, mail_verified FROM users WHERE mail= ?");
        $stmt->execute(array($email));

        return $stmt->fetch();
    }

    public static function validate($name, $mail, $password, $sector)
    {
        if($name == null || $mail == null || $password == null || $sector == null)
        {
            return false;
        }
        return true;
    }

    public static function login(Connection $connection,$email, $password)
    {
        $con = $connection->get_connection();

        $stmt = $con->prepare("SELECT * FROM users WHERE mail= ?");
        $stmt->execute(array($email));
        $user = $stmt->fetch();

        if($user && password_verify($password, $user['password']))
        {
            return true;
        }
        return false;
    }

    public static function actualizar(Connection $connection, $name, $mail, $sector, $id)
    {
        $con = $connection->get_connection();

        $name = self::escapeData($name);
        $mail = self::escapeData($mail);
        $sector = self::escapeData($sector);
        $id = self::escapeData($id);

        $stmt = $con->prepare("UPDATE users SET name=:name, mail=:mail, sector=:sector WHERE id=:id");
        $stmt->bindParam(':name', $name, PDO::PARAM_STR);
        $stmt->bindParam(':mail', $mail, PDO::PARAM_STR);
        $stmt->bindParam(':sector', $sector, PDO::PARAM_STR);
        $stmt->bindParam(':id', $id, PDO::PARAM_STR);
        return $stmt->execute();
    }

    public static function verifyEmail(Connection $connection, $token)
    {
        $con = $connection->get_connection();
        $stmt = $con->prepare("UPDATE users SET mail_verified=1 WHERE token=:token");
        $stmt->bindParam(':token', $token, PDO::PARAM_STR);
        $stmt->execute();

        $con = $connection->get_connection();
        $stmt = $con->prepare("SELECT mail FROM users WHERE token= ?");
        $stmt->execute(array($token));

        return $stmt->fetch();
    }

    protected static function generateRandomString($length = 10) {
        $chars = '0123456789abcdefghijklmnñopqrstuvwxyzABCDEFGHIJKLMNÑOPQRSTUVWXYZ!?#_-';
        $charactersLength = strlen($chars);
        $randomString = '';
        for ($i = 0; $i < $length; $i++) {
            $randomString .= $chars[rand(0, $charactersLength - 1)];
        }
        return $randomString;
    }

    public function getToken()
    {
        return $this->token;
    }
}